@@ -1,5 +1,9 @@
-import sys, os, __builtin__
-from mercurial import util
+from __future__ import absolute_import
+import __builtin__
+import os
+from mercurial import (
+    util,
+)
 
 def lowerwrap(scope, funcname):
     f = getattr(scope, funcname)
@@ -7,7 +11,7 @@ def lowerwrap(scope, funcname):
         d, base = os.path.split(fname)
         try:
             files = os.listdir(d or '.')
-        except OSError, inst:
+        except OSError:
             files = []
         if base in files:
             return f(fname, *args, **kwargs)
