@@ -19,16 +19,19 @@
 //! 'Generic DAG ancestor algorithms - Rust implementation'
 //! ```
 
+/// This crate uses nested private macros, `extern crate` is still needed in
+/// 2018 edition.
 #[macro_use]
 extern crate cpython;
-extern crate hg;
-extern crate libc;
 
 pub mod ancestors;
 mod cindex;
 mod conversion;
 pub mod dagops;
+pub mod dirstate;
+pub mod discovery;
 pub mod exceptions;
+pub mod filepatterns;
 
 py_module_initializer!(rustext, initrustext, PyInit_rustext, |py, m| {
     m.add(
@@ -40,6 +43,23 @@ py_module_initializer!(rustext, initrustext, PyInit_rustext, |py, m| {
     let dotted_name: String = m.get(py, "__name__")?.extract(py)?;
     m.add(py, "ancestor", ancestors::init_module(py, &dotted_name)?)?;
     m.add(py, "dagop", dagops::init_module(py, &dotted_name)?)?;
+    m.add(py, "discovery", discovery::init_module(py, &dotted_name)?)?;
+    m.add(py, "dirstate", dirstate::init_module(py, &dotted_name)?)?;
+    m.add(
+        py,
+        "filepatterns",
+        filepatterns::init_module(py, &dotted_name)?,
+    )?;
     m.add(py, "GraphError", py.get_type::<exceptions::GraphError>())?;
+    m.add(
+        py,
+        "PatternFileError",
+        py.get_type::<exceptions::PatternFileError>(),
+    )?;
+    m.add(
+        py,
+        "PatternError",
+        py.get_type::<exceptions::PatternError>(),
+    )?;
     Ok(())
 });
